using System.Threading;
using System.Security.Principal;
using System.Security.AccessControl;
public class SingleGlobalInstance : IDisposable
{
    bool hasHandle;
    Mutex mutex;

    public SingleGlobalInstance(string mutexName, int timeOut)
    {
        string mutexId = string.Format("Global\\{{{0}}}", mutexName);
        mutex = new Mutex(false, mutexId);

        var allowEveryoneRule = new MutexAccessRule(new SecurityIdentifier(WellKnownSidType.WorldSid, null), MutexRights.FullControl, AccessControlType.Allow);
        var securitySettings = new MutexSecurity();
        securitySettings.AddAccessRule(allowEveryoneRule);
        mutex.SetAccessControl(securitySettings);
         try
        {
            if(timeOut <= 0)
                hasHandle = mutex.WaitOne(Timeout.Infinite, false);
            else
                hasHandle = mutex.WaitOne(timeOut, false);

            if (hasHandle == false)
                throw new TimeoutException("Another build is running. Please wait.");
        }
        catch (AbandonedMutexException)
        {
            hasHandle = true;
        }
    }


    public void Dispose()
    {
        if (hasHandle && mutex != null)
            mutex.ReleaseMutex();
    }
}
